resource "aws_acm_certificate" "certificate" {
  domain_name       = "api.dev.postgrescheck.com"
  validation_method = "DNS"

  lifecycle {
    create_before_destroy = true
  }

  provider = aws.useast1
}

resource "aws_route53_record" "validation_records" {
  for_each = {
    for dvo in aws_acm_certificate.certificate.domain_validation_options : dvo.domain_name => {
      name   = dvo.resource_record_name
      record = dvo.resource_record_value
      type   = dvo.resource_record_type
    }
  }

  allow_overwrite = true
  name            = each.value.name
  records         = [each.value.record]
  ttl             = 60
  type            = each.value.type
  zone_id         = "Z04292923K6NFXDJ6OZ9U"
}

resource "aws_acm_certificate_validation" "validation" {
  certificate_arn         = aws_acm_certificate.certificate.arn
  validation_record_fqdns = [for record in aws_route53_record.validation_records : record.fqdn]
  depends_on              = [aws_acm_certificate.certificate]
  provider                = aws.useast1
}

resource "aws_iam_role" "lambda_role" {
  name               = "postgres-check-api-lambda-role"
  assume_role_policy = jsonencode({
    "Version" : "2012-10-17",
    "Statement" : [
      {
        "Action" : "sts:AssumeRole",
        "Principal" : {
          "Service" : "lambda.amazonaws.com"
        },
        "Effect" : "Allow"
      }
    ]
  })
}

resource "aws_iam_policy" "lambda_policy" {
  name   = "postgres-check-api-lambda-policy"
  path   = "/"
  policy = jsonencode(
    {
      "Version" : "2012-10-17",
      "Statement" : [
        {
          "Action" : [
            "logs:CreateLogGroup",
            "logs:CreateLogStream",
            "logs:PutLogEvents"
          ],
          "Resource" : "arn:aws:logs:*:*:*",
          "Effect" : "Allow"
        },
        {
          "Action" : [
            "dynamodb:*"
          ],
          "Resource" : [
            "arn:aws:dynamodb:eu-west-1:211125682198:table/postgres-check",
            "arn:aws:dynamodb:eu-west-1:211125682198:table/postgres-check/*"
          ],
          "Effect" : "Allow"
        }
      ]
    })
}

resource "aws_iam_role_policy_attachment" "lambda_role_policy_attachment" {
  role       = aws_iam_role.lambda_role.name
  policy_arn = aws_iam_policy.lambda_policy.arn
}

data "archive_file" "lambda_code" {
  type        = "zip"
  source_dir  = "${path.module}/../build/"
  output_path = "${path.module}/lambda-code.zip"
}

resource "aws_lambda_function" "lambda_function" {
  filename         = data.archive_file.lambda_code.output_path
  function_name    = "postgres-check-api-lambda"
  role             = aws_iam_role.lambda_role.arn
  handler          = "index.handler"
  runtime          = "nodejs20.x"
  depends_on       = [aws_iam_role_policy_attachment.lambda_role_policy_attachment, data.archive_file.lambda_code]
  source_code_hash = data.archive_file.lambda_code.output_base64sha256
}

resource "aws_api_gateway_domain_name" "domain" {
  domain_name     = "api.dev.postgrescheck.com"
  certificate_arn = aws_acm_certificate_validation.validation.certificate_arn
}

resource "aws_route53_record" "gateway_record" {
  name    = aws_api_gateway_domain_name.domain.domain_name
  type    = "A"
  zone_id = "Z04292923K6NFXDJ6OZ9U"

  alias {
    evaluate_target_health = true
    name                   = aws_api_gateway_domain_name.domain.cloudfront_domain_name
    zone_id                = aws_api_gateway_domain_name.domain.cloudfront_zone_id
  }
}

resource "aws_api_gateway_rest_api" "api" {
  name = "postgres-check-api"
  body = templatefile("${path.module}/../generated/postgres-check-api/schema.json", {
    lambda: aws_lambda_function.lambda_function.invoke_arn
  })
}



resource "aws_api_gateway_deployment" "stage" {
  rest_api_id = aws_api_gateway_rest_api.api.id
  stage_name  = "v1"
  triggers = {
    redeployment = sha1(jsonencode(aws_api_gateway_rest_api.api.body))
  }
}

resource "aws_lambda_permission" "lambda_permission" {
  statement_id  = "AllowAPIGatewayInvoke"
  action        = "lambda:InvokeFunction"
  function_name = aws_lambda_function.lambda_function.function_name
  principal     = "apigateway.amazonaws.com"
  source_arn    = "${aws_api_gateway_rest_api.api.execution_arn}/*"
}

resource "aws_api_gateway_base_path_mapping" "mapping" {
  api_id      = aws_api_gateway_rest_api.api.id
  stage_name = aws_api_gateway_deployment.stage.stage_name
  domain_name = aws_api_gateway_domain_name.domain.domain_name
}